package eu.mhsl.craftattack.spawn;

import eu.mhsl.craftattack.spawn.appliance.Appliance;
import eu.mhsl.craftattack.spawn.appliances.adminMarker.AdminMarker;
import eu.mhsl.craftattack.spawn.appliances.chatMessages.ChatMessages;
import eu.mhsl.craftattack.spawn.appliances.countdown.Countdown;
import eu.mhsl.craftattack.spawn.appliances.debug.Debug;
import eu.mhsl.craftattack.spawn.appliances.report.Report;
import eu.mhsl.craftattack.spawn.appliances.tablist.Tablist;
import eu.mhsl.craftattack.spawn.appliances.titleClear.TitleClear;
import eu.mhsl.craftattack.spawn.config.Configuration;
import eu.mhsl.craftattack.spawn.appliances.worldmuseum.WorldMuseum;
import org.bukkit.Bukkit;
import org.bukkit.event.HandlerList;
import org.bukkit.plugin.java.JavaPlugin;

import java.lang.reflect.ParameterizedType;
import java.util.List;

public final class Main extends JavaPlugin {
    private static Main instance;

    private List<Appliance> appliances;
    @Override
    public void onEnable() {
        instance = this;
        saveDefaultConfig();
        Configuration.readConfig();

        appliances = List.of(
                new AdminMarker(),
                new WorldMuseum(),
                new TitleClear(),
                new Countdown(),
                new Tablist(),
                new ChatMessages(),
                new Report(),
                new Debug()
        );
        Bukkit.getLogger().info("Loading appliances...");
        appliances.forEach(appliance -> {
            appliance.initialize(this);
            appliance.onEnable();
        });
        Bukkit.getLogger().info("Loaded " + appliances.size() + " appliances!");

        getServer().getMessenger().registerOutgoingPluginChannel(this, "BungeeCord");
    }

    @Override
    public void onDisable() {
        appliances.forEach(appliance -> {
            appliance.onDisable();
            appliance.destruct(this);
        });
        HandlerList.unregisterAll(this);
        Bukkit.getScheduler().cancelTasks(this);
    }

    public <T extends Appliance> T getAppliance(Class<T> clazz) {
        return this.appliances.stream().filter(clazz::isInstance).map(clazz::cast).findFirst().orElseThrow();
    }

    @SuppressWarnings("unchecked")
    public static <T> Class<T> getApplianceType(Class<?> clazz) {
        return (Class<T>) ((ParameterizedType) clazz.getGenericSuperclass()).getActualTypeArguments()[0];
    }


    public static Main instance() {
        return instance;
    }
}
