package eu.mhsl.craftattack.spawn.appliances.countdown;

import eu.mhsl.craftattack.spawn.Main;
import eu.mhsl.craftattack.spawn.appliance.Appliance;
import eu.mhsl.craftattack.spawn.appliance.ApplianceCommand;
import eu.mhsl.craftattack.spawn.appliances.countdown.command.ProjectStartCancelCommand;
import eu.mhsl.craftattack.spawn.appliances.countdown.command.ProjectStartCommand;
import eu.mhsl.craftattack.spawn.appliances.countdown.command.ProjectStartResetCommand;
import eu.mhsl.craftattack.spawn.appliances.countdown.listener.PlayerInvincibleListener;
import eu.mhsl.craftattack.spawn.config.Configuration;
import net.kyori.adventure.text.format.NamedTextColor;
import org.bukkit.Bukkit;
import org.bukkit.WorldBorder;
import org.bukkit.event.Listener;
import org.jetbrains.annotations.NotNull;

import java.util.List;
import java.util.function.Consumer;

public class Countdown extends Appliance {
    public Countdown() {
        super("countdown");
    }

    private int taskId = -1;
    private boolean isRunning = false;
    private int countDown;
    public void startCountdown() {
        if(!isEnabled()) return;
        if(isRunning) return;

        this.countDown = localConfig().getInt("countdown");
        this.isRunning = true;
        this.taskId = Bukkit.getScheduler().scheduleSyncRepeatingTask(Main.instance(), () -> {
            if(this.countDown <= 0) {
                this.cancelCountdown();
                this.projectStart();
            }

            Bukkit.getOnlinePlayers().forEach(player -> player.sendMessage("Sekunden:" + this.countDown));
            this.countDown--;
        }, 20, 20);
    }

    public void cancelCountdown() {
        if(taskId == -1) return;
        Bukkit.getScheduler().cancelTask(this.taskId);
        this.isRunning = false;
        this.restoreBeforeStart();
    }

    public boolean isEnabled() {
        return localConfig().getBoolean("enabled");
    }

    public void setEnabled(boolean enabled) {
        localConfig().set("enabled", enabled);
        Configuration.saveChanges();
    }

    public void projectStart() {
        setEnabled(false);

        worldBorderModifier(worldBorder -> worldBorder.setSize(worldBorder.getMaxSize()));
        Bukkit.getOnlinePlayers().forEach(player -> {
            player.setFoodLevel(20);
            player.setHealth(20);
            player.playerListName(player.playerListName().color(NamedTextColor.AQUA));
        });
    }

    public void restoreBeforeStart() {
        setEnabled(true);

        worldBorderModifier(worldBorder -> {
            worldBorder.setSize(localConfig().getLong("worldborder-before"));
            worldBorder.setWarningDistance(0);
            worldBorder.setDamageAmount(0);
        });
    }

    private void worldBorderModifier(Consumer<WorldBorder> worldBorder) {
        Bukkit.getWorlds().forEach(world -> worldBorder.accept(world.getWorldBorder()));
    }

    public boolean isRunning() {
        return isRunning;
    }

    @Override
    protected @NotNull List<Listener> eventHandlers() {
        return List.of(new PlayerInvincibleListener());
    }

    @Override
    protected @NotNull List<ApplianceCommand<?>> commands() {
        return List.of(
                new ProjectStartCommand(),
                new ProjectStartCancelCommand(),
                new ProjectStartResetCommand()
        );
    }
}
