package eu.mhsl.craftattack.spawn.appliances.worldmuseum;

import eu.mhsl.craftattack.spawn.appliance.Appliance;
import eu.mhsl.craftattack.spawn.appliance.ApplianceCommand;
import eu.mhsl.craftattack.spawn.appliances.worldmuseum.listener.InventoryOpenListener;
import eu.mhsl.craftattack.spawn.config.ConfigUtil;
import eu.mhsl.craftattack.spawn.config.Configuration;
import eu.mhsl.craftattack.spawn.appliances.worldmuseum.command.MoveWorldMuseumVillagerCommand;
import eu.mhsl.craftattack.spawn.appliances.worldmuseum.listener.PlayerEntityInteractListener;
import eu.mhsl.craftattack.spawn.util.DisplayVillager;
import eu.mhsl.craftattack.spawn.util.PluginMessage;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.format.NamedTextColor;
import org.bukkit.Bukkit;
import org.bukkit.Location;
import org.bukkit.entity.*;
import org.bukkit.event.Listener;
import org.jetbrains.annotations.NotNull;

import java.util.List;
import java.util.Objects;
import java.util.UUID;

public class WorldMuseum extends Appliance {
    public DisplayVillager villager;

    public WorldMuseum() {
        super("worldMuseum");
    }

    @Override
    public void onEnable() {
        Location location = ConfigUtil.Position.paseLocation(Objects.requireNonNull(localConfig().getConfigurationSection("villagerLocation")));
        this.villager = new DisplayVillager(
                UUID.fromString(localConfig().getString("uuid", UUID.randomUUID().toString())),
                location,
                villager -> {
                    localConfig().set("uuid", villager.getUniqueId().toString());
                    Configuration.saveChanges();

                    villager.customName(Component.text("Weltenansicht").color(NamedTextColor.GOLD));
                    villager.setProfession(Villager.Profession.LIBRARIAN);
                    villager.setVillagerType(Villager.Type.SAVANNA);
                }
        );
    }
    public void updateVillagerPosition(Location location) {
        ConfigUtil.Position.writeLocation(
                Objects.requireNonNull(localConfig().getConfigurationSection("villagerLocation")),
                location
        );
        Configuration.saveChanges();

        this.villager.getVillager().teleport(location);
    }

    public void handleVillagerInteraction(Player player) {
        Bukkit.getLogger().info("Sending" + player.getName() + " to WorldMuseum");
        PluginMessage.connect(player, localConfig().getString("connect-server-name"));
    }

    @Override
    protected @NotNull List<ApplianceCommand<?>> commands() {
        return List.of(new MoveWorldMuseumVillagerCommand());
    }

    @Override
    protected @NotNull List<Listener> eventHandlers() {
        return List.of(
                new PlayerEntityInteractListener(),
                new InventoryOpenListener()
        );
    }
}
